@@ -23,10 +23,6 @@ require 'enumerator'
 # @author Willem van Bergen
 module ChunkyPNG
 
-  # The current version of ChunkyPNG. This value will be updated 
-  # automatically by them <tt>gem:release</tt> rake task.
-  VERSION = "1.2.6"
-
   ###################################################
   # PNG international standard defined constants
   ###################################################
@@ -124,19 +120,29 @@ module ChunkyPNG
   # Exception that is raised if an expectation fails.
   class OutOfBounds < ChunkyPNG::ExpectationFailed
   end
+
+  # Exception that is raised when requesting the DPI of a PNG that doesn't
+  # specify the units of its physical pixel dimensions.
+  class UnitsUnknown < ChunkyPNG::Exception
+  end
+
+  def self.force_binary(str)
+    str.respond_to?(:force_encoding) ? str.force_encoding('BINARY') : str
+  end
   
   # Empty byte array. This basically is an empty string, but with the encoding
   # set correctly to ASCII-8BIT (binary) in Ruby 1.9.
   # @return [String] An empty string, with encoding set to binary in Ruby 1.9
   # @private
-  EMPTY_BYTEARRAY = String.method_defined?(:force_encoding) ? "".force_encoding('ASCII-8BIT').freeze : "".freeze
+  EMPTY_BYTEARRAY = force_binary(String.new).freeze
 
   # Null-byte, with the encoding set correctly to ASCII-8BIT (binary) in Ruby 1.9.
   # @return [String] A binary string, consisting of one NULL-byte. 
   # @private
-  EXTRA_BYTE = String.method_defined?(:force_encoding) ? "\0".force_encoding('ASCII-8BIT') : "\0"
+  EXTRA_BYTE = force_binary(String.new("\0")).freeze
 end
 
+require 'chunky_png/version'
 
 # Ruby 1.8 / 1.9 compatibility
 require 'chunky_png/compatibility'
